@@ -118,7 +118,7 @@ CResultTileFx::CResultTileFx()
       HERE(),
       nullptr,
       array_->array_->array_schema_latest_ptr(),
-      URI(),
+      generate_fragment_uri(array_->array_.get()),
       std::make_pair<uint64_t, uint64_t>(0, 0),
       memory_tracker_,
       false);
@@ -198,7 +198,7 @@ TEST_CASE_METHOD(
   FragmentMetadata frag_md(
       nullptr,
       array_->array_->array_schema_latest_ptr(),
-      URI(),
+      generate_fragment_uri(array_->array_.get()),
       std::make_pair<uint64_t, uint64_t>(0, 0),
       memory_tracker_,
       true);
@@ -283,7 +283,7 @@ TEST_CASE_METHOD(
   std::iota(range_indexes.begin(), range_indexes.end(), 0);
 
   auto resource = tiledb::test::get_test_memory_tracker()->get_resource(
-      MemoryType::TILE_BITMAP);
+      MemoryType::RESULT_TILE_BITMAP);
   tdb::pmr::vector<uint8_t> result_count(num_cells, 1, resource);
   ResultTile::compute_results_count_sparse_string(
       &rt,
@@ -311,7 +311,7 @@ TEST_CASE_METHOD(
   FragmentMetadata frag_md(
       nullptr,
       array_->array_->array_schema_latest_ptr(),
-      URI(),
+      generate_fragment_uri(array_->array_.get()),
       std::make_pair<uint64_t, uint64_t>(0, 0),
       memory_tracker_,
       true);
@@ -426,7 +426,7 @@ TEST_CASE_METHOD(
   std::iota(range_indexes.begin(), range_indexes.end(), 0);
 
   auto resource = tiledb::test::get_test_memory_tracker()->get_resource(
-      MemoryType::TILE_BITMAP);
+      MemoryType::RESULT_TILE_BITMAP);
   tdb::pmr::vector<uint64_t> result_count(num_cells, 1, resource);
   ResultTile::compute_results_count_sparse_string(
       &rt,
