@@ -214,7 +214,10 @@ class ArraySchema {
   }
 
   /** Returns the attributes. */
-  const tdb::pmr::vector<shared_ptr<const Attribute>>& attributes() const;
+  inline const tdb::pmr::vector<shared_ptr<const Attribute>>& attributes()
+      const {
+    return attributes_;
+  }
 
   /** Returns the capacity. */
   uint64_t capacity() const;
@@ -265,6 +268,11 @@ class ArraySchema {
   /** Return the pipeline used for coordinates. */
   const FilterPipeline& coords_filters() const;
 
+  /** Return the array current domain. */
+  inline const CurrentDomain& current_domain() const {
+    return *current_domain_;
+  }
+
   /** True if the array is dense. */
   bool dense() const;
 
@@ -299,9 +307,6 @@ class ArraySchema {
   /** Returns the number of dimensions. */
   dimension_size_type dim_num() const;
 
-  /** Dumps the array schema in ASCII format in the selected output. */
-  void dump(FILE* out) const;
-
   /**
    * Checks if the array schema has a attribute of the given name.
    *
@@ -329,14 +334,6 @@ class ArraySchema {
   /** Returns true if the input name is nullable. */
   bool is_nullable(const std::string& name) const;
 
-  /**
-   * Serializes the array schema object into a buffer.
-   *
-   * @param serializer The object the array schema is serialized into.
-   * @return Status
-   */
-  void serialize(Serializer& serializer) const;
-
   /** Returns the tile order. */
   Layout tile_order() const;
 
@@ -588,6 +585,25 @@ class ArraySchema {
       std::optional<std::pair<uint64_t, uint64_t>> timestamp_range =
           std::nullopt);
 
+  /** Returns the enumeration map. */
+  inline const tdb::pmr::
+      unordered_map<std::string, shared_ptr<const Enumeration>>&
+      enumeration_map() const {
+    return enumeration_map_;
+  }
+
+  /** Returns the enumeration path map. */
+  inline const tdb::pmr::unordered_map<std::string, std::string>&
+  enumeration_path_map() const {
+    return enumeration_path_map_;
+  }
+
+  /** Returns the dimension labels. */
+  inline const tdb::pmr::vector<shared_ptr<const DimensionLabel>>&
+  dimension_labels() const {
+    return dimension_labels_;
+  }
+
   /**
    * Expand the array current domain
    *
@@ -708,7 +724,7 @@ class ArraySchema {
   tdb::pmr::unordered_map<std::string, shared_ptr<const Enumeration>>
       enumeration_map_;
 
-  /** A map of Enumeration names to Enumeration URIs */
+  /** A map of Enumeration names to Enumeration filenames */
   tdb::pmr::unordered_map<std::string, std::string> enumeration_path_map_;
 
   /** The filter pipeline run on offset tiles for var-length attributes. */
@@ -768,4 +784,8 @@ class ArraySchema {
 
 }  // namespace tiledb::sm
 
+/** Converts the filter into a string representation. */
+std::ostream& operator<<(
+    std::ostream& os, const tiledb::sm::ArraySchema& schema);
+
 #endif  // TILEDB_ARRAY_SCHEMA_H
