@@ -269,9 +269,6 @@ class Domain {
    */
   shared_ptr<Dimension> shared_dimension(const std::string& name) const;
 
-  /** Dumps the domain in ASCII format in the selected output. */
-  void dump(FILE* out) const;
-
   /** Expands ND range `r2` using ND range `r1`. */
   void expand_ndrange(const NDRange& r1, NDRange* r2) const;
 
@@ -702,4 +699,7 @@ class Domain {
 
 }  // namespace tiledb::sm
 
+/** Converts the filter into a string representation. */
+std::ostream& operator<<(std::ostream& os, const tiledb::sm::Domain& domain);
+
 #endif  // TILEDB_DOMAIN_H
