@@ -126,12 +126,6 @@ class FilterPipeline {
   static FilterPipeline deserialize(
       Deserializer& deserializer, const uint32_t version, Datatype datatype);
 
-  /**
-   * Dumps the filter pipeline details in ASCII format in the selected
-   * output.
-   */
-  void dump(FILE* out) const;
-
   /**
    * Checks that two filters have compatible input / output types.
    * Checks fail if the first filter outputs a type not accepted by the second
@@ -318,6 +312,8 @@ class FilterPipeline {
   bool use_tile_chunking(
       const bool is_var, const uint32_t version, const Datatype type) const;
 
+  std::vector<shared_ptr<Filter>> filters() const;
+
  private:
   /** A pair of FilterBuffers. */
   typedef std::pair<FilterBuffer, FilterBuffer> FilterBufferPair;
@@ -372,3 +368,7 @@ class FilterPipeline {
 }  // namespace tiledb::sm
 
 #endif  // TILEDB_FILTER_PIPELINE_H
+
+/** Converts the filter into a string representation. */
+std::ostream& operator<<(
+    std::ostream& os, const tiledb::sm::FilterPipeline& filter_pipeline);
