@@ -35,7 +35,6 @@
 #include "tiledb/sm/filesystem/ssl_config.h"
 #include "tiledb/sm/filesystem/uri.h"
 #include "tiledb/sm/misc/tdb_time.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/stats/global_stats.h"
 
 #include <algorithm>
@@ -45,33 +44,12 @@
 
 #ifdef _WIN32
 #include <locale>  //provides needed visibility of two-argument tolower() prototype for win32
+#include "curl.h"
 #endif
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
-
-/**
- * Wraps opaque user data to be invoked with a write callback.
- */
-struct WriteCbState {
-  /** Default constructor. */
-  WriteCbState()
-      : reset(true)
-      , arg(NULL)
-      , skip_retries(false) {
-  }
-
-  /** True if this is the first write callback invoked in a request retry. */
-  bool reset;
-
-  /** The opaque user data to pass to the write callback. */
-  void* arg;
-
-  /** True if the internal curl retries should be skipped. */
-  bool skip_retries;
-};
+namespace tiledb::sm {
 
 /**
  * @brief Callback for saving unbuffered libcurl response data
@@ -240,10 +218,12 @@ size_t write_header_callback(
       const std::string header_value_domain =
           header_value_scheme_excl.substr(0, header_domain_end_pos);
 
-      const std::string redirection_value =
-          header_scheme + "://" + header_value_domain;
-      std::unique_lock<std::mutex> rd_lck(*(pmHeader->redirect_uri_map_lock));
-      (*pmHeader->redirect_uri_map)[*pmHeader->uri] = redirection_value;
+      if (pmHeader->should_cache_redirect) {
+        const std::string redirection_value =
+            header_scheme + "://" + header_value_domain;
+        std::unique_lock<std::mutex> rd_lck(*(pmHeader->redirect_uri_map_lock));
+        (*pmHeader->redirect_uri_map)[*pmHeader->uri] = redirection_value;
+      }
     }
   }
 
@@ -257,14 +237,16 @@ Curl::Curl(const std::shared_ptr<Logger>& logger)
     , retry_delay_factor_(0)
     , retry_initial_delay_ms_(0)
     , logger_(logger->clone("curl ", ++logger_id_))
-    , verbose_(false) {
+    , verbose_(false)
+    , retry_curl_errors_(true) {
 }
 
 Status Curl::init(
     const Config* config,
     const std::unordered_map<std::string, std::string>& extra_headers,
     std::unordered_map<std::string, std::string>* const res_headers,
-    std::mutex* const res_mtx) {
+    std::mutex* const res_mtx,
+    bool should_cache_redirect) {
   if (config == nullptr)
     return LOG_STATUS(
         Status_RestError("Error initializing libcurl; config is null."));
@@ -274,6 +256,7 @@ Status Curl::init(
   extra_headers_ = extra_headers;
   headerData.redirect_uri_map = res_headers;
   headerData.redirect_uri_map_lock = res_mtx;
+  headerData.should_cache_redirect = should_cache_redirect;
 
   // See https://curl.haxx.se/libcurl/c/threadsafe.html
   CURLcode rc = curl_easy_setopt(curl_.get(), CURLOPT_NOSIGNAL, 1);
@@ -341,6 +324,9 @@ Status Curl::init(
       "rest.curl.buffer_size", &curl_buffer_size_, &found));
   assert(found);
 
+  retry_curl_errors_ =
+      config_->get<bool>("rest.curl.retry_errors", Config::must_find);
+
   return Status::Ok();
 }
 
@@ -427,11 +413,13 @@ Status Curl::make_curl_request(
     stats::Stats* const stats,
     const char* url,
     CURLcode* curl_code,
+    BufferList* data,
     Buffer* returned_data) const {
   return make_curl_request_common(
       stats,
       url,
       curl_code,
+      data,
       &write_memory_callback,
       static_cast<void*>(returned_data));
 }
@@ -440,11 +428,13 @@ Status Curl::make_curl_request(
     stats::Stats* const stats,
     const char* url,
     CURLcode* curl_code,
+    BufferList* data,
     PostResponseCb&& cb) const {
   return make_curl_request_common(
       stats,
       url,
       curl_code,
+      data,
       &write_memory_callback_cb,
       static_cast<void*>(&cb));
 }
@@ -453,9 +443,9 @@ CURLcode Curl::curl_easy_perform_instrumented(
     const char* const url, const uint8_t retry_number) const {
   CURL* curl = curl_.get();
   // Time the curl transfer
-  uint64_t t1 = tiledb::sm::utils::time::timestamp_now_ms();
+  uint64_t t1 = utils::time::timestamp_now_ms();
   auto curl_code = curl_easy_perform(curl);
-  uint64_t t2 = tiledb::sm::utils::time::timestamp_now_ms();
+  uint64_t t2 = utils::time::timestamp_now_ms();
   uint64_t dt = t2 - t1;
   long http_code = 0;
   if (curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_code) != CURLE_OK) {
@@ -476,10 +466,65 @@ CURLcode Curl::curl_easy_perform_instrumented(
   return curl_code;
 }
 
+void Curl::set_curl_request_options(
+    const char* const url,
+    size_t (*write_cb)(void*, size_t, size_t, void*),
+    WriteCbState& write_cb_state) const {
+  CURL* curl = curl_.get();
+  if (curl == nullptr) {
+    throw std::runtime_error(
+        "Cannot make curl request; curl instance is null.");
+  }
+  /* set url to fetch */
+  curl_easy_setopt(curl, CURLOPT_URL, url);
+
+  /* set callback function */
+  curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_cb);
+
+  /* pass fetch buffer pointer */
+  curl_easy_setopt(
+      curl, CURLOPT_WRITEDATA, static_cast<void*>(&write_cb_state));
+
+  /* Set curl verbose mode */
+  curl_easy_setopt(curl, CURLOPT_VERBOSE, verbose_);
+
+  /* set compression */
+  const char* compressor = nullptr;
+  throw_if_not_ok(config_->get("rest.http_compressor", &compressor));
+
+  if (compressor != nullptr) {
+    // curl expects lowecase strings so let's convert
+    std::string comp(compressor);
+    std::locale loc;
+    for (std::string::size_type j = 0; j < comp.length(); ++j)
+      comp[j] = std::tolower(comp[j], loc);
+
+    if (comp != "none") {
+      if (comp == "any") {
+        comp = "";
+      }
+      curl_easy_setopt(curl, CURLOPT_ACCEPT_ENCODING, comp.c_str());
+    }
+  }
+
+  /* enable location redirects */
+  curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
+
+  /* set maximum allowed redirects */
+  curl_easy_setopt(curl, CURLOPT_MAXREDIRS, 1);
+
+  /* enable forwarding auth to redirects */
+  curl_easy_setopt(curl, CURLOPT_UNRESTRICTED_AUTH, 1L);
+
+  /* Set max buffer size */
+  curl_easy_setopt(curl, CURLOPT_BUFFERSIZE, curl_buffer_size_);
+}
+
 Status Curl::make_curl_request_common(
     stats::Stats* const stats,
     const char* const url,
     CURLcode* const curl_code,
+    BufferList* data,
     size_t (*write_cb)(void*, size_t, size_t, void*),
     void* const write_cb_arg) const {
   CURL* curl = curl_.get();
@@ -489,111 +534,70 @@ Status Curl::make_curl_request_common(
 
   *curl_code = CURLE_OK;
   uint64_t retry_delay = retry_initial_delay_ms_;
+
+  // Save the offsets before the request in case we need to retry
+  size_t current_buffer_index = 0;
+  uint64_t current_relative_offset = 0;
+  if (data != nullptr) {
+    std::tie(current_buffer_index, current_relative_offset) =
+        data->get_offset();
+  }
+
   // <= because the 0ths retry is actually the initial request
   for (uint8_t i = 0; i <= retry_count_; i++) {
     WriteCbState write_cb_state;
     write_cb_state.arg = write_cb_arg;
 
-    /* set url to fetch */
-    curl_easy_setopt(curl, CURLOPT_URL, url);
-
-    /* set callback function */
-    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_cb);
-
-    /* pass fetch buffer pointer */
-    curl_easy_setopt(
-        curl, CURLOPT_WRITEDATA, static_cast<void*>(&write_cb_state));
-
-    /* Set curl verbose mode */
-    curl_easy_setopt(curl, CURLOPT_VERBOSE, verbose_);
-
-    /* set default user agent */
-    // curl_easy_setopt(curl, CURLOPT_USERAGENT, "libcurl-agent/1.0");
-
-    /* Fail on error */
-    // curl_easy_setopt(curl, CURLOPT_FAILONERROR, 1L);
-
-    /* set timeout */
-    // curl_easy_setopt(curl, CURLOPT_TIMEOUT, 5);
-
-    /* set compression */
-    const char* compressor = nullptr;
-    RETURN_NOT_OK(config_->get("rest.http_compressor", &compressor));
-
-    if (compressor != nullptr) {
-      // curl expects lowecase strings so let's convert
-      std::string comp(compressor);
-      std::locale loc;
-      for (std::string::size_type j = 0; j < comp.length(); ++j)
-        comp[j] = std::tolower(comp[j], loc);
-
-      if (comp != "none") {
-        if (comp == "any") {
-          comp = "";
-        }
-        curl_easy_setopt(curl, CURLOPT_ACCEPT_ENCODING, comp.c_str());
-      }
+    // If this is a retry we need to reset the offsets in the data buffer list
+    // to the initial position before the failed request so that we send the
+    // correct data.
+    if (data != nullptr && retry_count_ > 0) {
+      data->set_offset(current_buffer_index, current_relative_offset);
     }
 
-    /* enable location redirects */
-    curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
+    // set the necessary curl options on the request
+    set_curl_request_options(url, write_cb, write_cb_state);
 
-    /* set maximum allowed redirects */
-    curl_easy_setopt(curl, CURLOPT_MAXREDIRS, 1);
+    /* perform the blocking network transfer */
+    CURLcode curl_code = curl_easy_perform_instrumented(url, i);
 
-    /* enable forwarding auth to redirects */
-    curl_easy_setopt(curl, CURLOPT_UNRESTRICTED_AUTH, 1L);
-
-    /* Set max buffer size */
-    curl_easy_setopt(curl, CURLOPT_BUFFERSIZE, curl_buffer_size_);
-
-    /* fetch the url */
-    CURLcode tmp_curl_code = curl_easy_perform_instrumented(url, i);
-
-    bool retry;
-    RETURN_NOT_OK(should_retry_based_on_http_status(&retry));
-
-    /* If Curl call was successful (not http status, but no socket error, etc)
-     * break */
-    if (tmp_curl_code == CURLE_OK && !retry) {
-      break;
-    }
-
-    /* If there is a write error we should not attempt a retry but instead
-     * return an error */
-    if (tmp_curl_code != CURLE_OK) {
-      /* Only store the first non-OK curl code, because it will likely be more
-       * useful than the curl codes from the retries. */
-      if (*curl_code == CURLE_OK) {
-        *curl_code = tmp_curl_code;
-      }
-
-      return Status_RestError(
-          "Curl error reading response from server: " +
-          get_curl_errstr(tmp_curl_code) + ".");
-    }
-
-    /* Retry on curl errors, unless the write callback has elected
-     * to skip it. */
-    if (write_cb_state.skip_retries) {
-      break;
-    }
-
-    // Only sleep if this isn't the last failed request allowed
-    if (i < retry_count_ - 1) {
-      long http_code = 0;
+    long http_code = 0;
+    if (curl_code == CURLE_OK) {
       if (curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_code) !=
-          CURLE_OK)
+          CURLE_OK) {
         return LOG_STATUS(Status_RestError(
             "Error checking curl error; could not get HTTP code."));
+      }
+    }
 
-      global_logger().debug(
-          "Request to {} failed with http response code {}, will sleep {}ms, "
-          "retry count {}",
-          url,
-          http_code,
-          retry_delay,
-          i);
+    // Exit if the request failed and we don't want to retry based on curl or
+    // HTTP code, or if the write callback has elected to skip retries
+    if (!should_retry_request(curl_code, http_code) ||
+        write_cb_state.skip_retries) {
+      break;
+    }
+
+    // Set up the actual retry logic
+    // Only sleep if this isn't the last failed request allowed
+    if (i < retry_count_ - 1) {
+      if (curl_code != CURLE_OK) {
+        global_logger().debug(
+            "Request to {} failed with Curl error message \"{}\", will sleep "
+            "{}ms, "
+            "retry count {}",
+            url,
+            get_curl_errstr(curl_code),
+            retry_delay,
+            i);
+      } else {
+        global_logger().debug(
+            "Request to {} failed with http response code {}, will sleep {}ms, "
+            "retry count {}",
+            url,
+            http_code,
+            retry_delay,
+            i);
+      }
       // Increment counter for number of retries
       stats->add_counter("rest_http_retries", 1);
       stats->add_counter("rest_http_retry_time", retry_delay);
@@ -607,29 +611,163 @@ Status Curl::make_curl_request_common(
   return Status::Ok();
 }
 
-Status Curl::should_retry_based_on_http_status(bool* retry) const {
-  // Set retry to false in case we get any errors from curl api calls
-  *retry = false;
-
-  CURL* curl = curl_.get();
-  if (curl == nullptr)
-    return LOG_STATUS(
-        Status_RestError("Error checking curl error; curl instance is null."));
-
-  long http_code = 0;
-  if (curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_code) != CURLE_OK)
-    return LOG_STATUS(Status_RestError(
-        "Error checking curl error; could not get HTTP code."));
-
+bool Curl::should_retry_based_on_http_status(long http_code) const {
   // Check http code for list of retries
   for (const auto& retry_code : retry_http_codes_) {
     if (http_code == (decltype(http_code))retry_code) {
-      *retry = true;
-      break;
+      return true;
     }
   }
 
-  return Status::Ok();
+  return false;
+}
+
+bool Curl::should_retry_based_on_curl_code(CURLcode curl_code) const {
+  if (!retry_curl_errors_) {
+    return false;
+  }
+
+  switch (curl_code) {
+    // Curl status of okay or non transient errors shouldn't be retried
+    case CURLE_OK:
+    case CURLE_URL_MALFORMAT:       /* 3 */
+    case CURLE_SSL_SHUTDOWN_FAILED: /* 80 - Failed to shut down the SSL
+                                      connection */
+    case CURLE_AUTH_ERROR: /* 94 - an authentication function returned an
+                             error */
+      return false;
+    case CURLE_UNSUPPORTED_PROTOCOL:  /* 1 */
+    case CURLE_FAILED_INIT:           /* 2 */
+    case CURLE_NOT_BUILT_IN:          /* 4 - [was obsoleted in August 2007 for
+                                        7.17.0, reused in April 2011 for 7.21.5] */
+    case CURLE_COULDNT_RESOLVE_PROXY: /* 5 */
+    case CURLE_COULDNT_RESOLVE_HOST:  /* 6 */
+    case CURLE_COULDNT_CONNECT:       /* 7 */
+    case CURLE_WEIRD_SERVER_REPLY:    /* 8 */
+    case CURLE_REMOTE_ACCESS_DENIED:  /* 9 a service was denied by the server
+                                        due to lack of access - when login fails
+                                        this is not returned. */
+    case CURLE_FTP_ACCEPT_FAILED:     /* 10 - [was obsoleted in April 2006 for
+                                        7.15.4, reused in Dec 2011 for 7.24.0]*/
+    case CURLE_FTP_WEIRD_PASS_REPLY:  /* 11 */
+    case CURLE_FTP_ACCEPT_TIMEOUT:    /* 12 - timeout occurred accepting server
+                                        [was obsoleted in August 2007 for 7.17.0,
+                                        reused in Dec 2011 for 7.24.0]*/
+    case CURLE_FTP_WEIRD_PASV_REPLY:  /* 13 */
+    case CURLE_FTP_WEIRD_227_FORMAT:  /* 14 */
+    case CURLE_FTP_CANT_GET_HOST:     /* 15 */
+    case CURLE_HTTP2: /* 16 - A problem in the http2 framing layer.
+                        [was obsoleted in August 2007 for 7.17.0,
+                        reused in July 2014 for 7.38.0] */
+    case CURLE_FTP_COULDNT_SET_TYPE:   /* 17 */
+    case CURLE_PARTIAL_FILE:           /* 18 */
+    case CURLE_FTP_COULDNT_RETR_FILE:  /* 19 */
+    case CURLE_OBSOLETE20:             /* 20 - NOT USED */
+    case CURLE_QUOTE_ERROR:            /* 21 - quote command failure */
+    case CURLE_HTTP_RETURNED_ERROR:    /* 22 */
+    case CURLE_WRITE_ERROR:            /* 23 */
+    case CURLE_OBSOLETE24:             /* 24 - NOT USED */
+    case CURLE_UPLOAD_FAILED:          /* 25 - failed upload "command" */
+    case CURLE_READ_ERROR:             /* 26 - couldn't open/read from file */
+    case CURLE_OUT_OF_MEMORY:          /* 27 */
+    case CURLE_OPERATION_TIMEDOUT:     /* 28 - the timeout time was reached */
+    case CURLE_OBSOLETE29:             /* 29 - NOT USED */
+    case CURLE_FTP_PORT_FAILED:        /* 30 - FTP PORT operation failed */
+    case CURLE_FTP_COULDNT_USE_REST:   /* 31 - the REST command failed */
+    case CURLE_OBSOLETE32:             /* 32 - NOT USED */
+    case CURLE_RANGE_ERROR:            /* 33 - RANGE "command" didn't work */
+    case CURLE_HTTP_POST_ERROR:        /* 34 */
+    case CURLE_SSL_CONNECT_ERROR:      /* 35 - wrong when connecting with SSL */
+    case CURLE_BAD_DOWNLOAD_RESUME:    /* 36 - couldn't resume download */
+    case CURLE_FILE_COULDNT_READ_FILE: /* 37 */
+    case CURLE_LDAP_CANNOT_BIND:       /* 38 */
+    case CURLE_LDAP_SEARCH_FAILED:     /* 39 */
+    case CURLE_OBSOLETE40:             /* 40 - NOT USED */
+    case CURLE_FUNCTION_NOT_FOUND:     /* 41 - NOT USED starting with 7.53.0 */
+    case CURLE_ABORTED_BY_CALLBACK:    /* 42 */
+    case CURLE_BAD_FUNCTION_ARGUMENT:  /* 43 */
+    case CURLE_OBSOLETE44:             /* 44 - NOT USED */
+    case CURLE_INTERFACE_FAILED:       /* 45 - CURLOPT_INTERFACE failed */
+    case CURLE_OBSOLETE46:             /* 46 - NOT USED */
+    case CURLE_TOO_MANY_REDIRECTS:     /* 47 - catch endless re-direct loops */
+    case CURLE_UNKNOWN_OPTION:       /* 48 - User specified an unknown option */
+    case CURLE_SETOPT_OPTION_SYNTAX: /* 49 - Malformed setopt option */
+    case CURLE_OBSOLETE50:           /* 50 - NOT USED */
+    case CURLE_OBSOLETE51:           /* 51 - NOT USED */
+    case CURLE_GOT_NOTHING:          /* 52 - when this is a specific error */
+    case CURLE_SSL_ENGINE_NOTFOUND:  /* 53 - SSL crypto engine not found */
+    case CURLE_SSL_ENGINE_SETFAILED: /* 54 - can not set SSL crypto engine as
+                                       default */
+    case CURLE_SEND_ERROR:           /* 55 - failed sending network data */
+    case CURLE_RECV_ERROR:      /* 56 - failure in receiving network data */
+    case CURLE_OBSOLETE57:      /* 57 - NOT IN USE */
+    case CURLE_SSL_CERTPROBLEM: /* 58 - problem with the local certificate */
+    case CURLE_SSL_CIPHER:      /* 59 - couldn't use specified cipher */
+    case CURLE_PEER_FAILED_VERIFICATION: /* 60 - peer's certificate or
+                                           fingerprint wasn't verified fine */
+    case CURLE_BAD_CONTENT_ENCODING:     /* 61 - Unrecognized/bad encoding */
+    case CURLE_OBSOLETE62:               /* 62 - NOT IN USE since 7.82.0 */
+    case CURLE_FILESIZE_EXCEEDED:        /* 63 - Maximum file size exceeded */
+    case CURLE_USE_SSL_FAILED:        /* 64 - Requested FTP SSL level failed */
+    case CURLE_SEND_FAIL_REWIND:      /* 65 - Sending the data requires a rewind
+                                        that failed */
+    case CURLE_SSL_ENGINE_INITFAILED: /* 66 - failed to initialise ENGINE */
+    case CURLE_LOGIN_DENIED:          /* 67 - user, password or similar was not
+                                        accepted and we failed to login */
+    case CURLE_TFTP_NOTFOUND:         /* 68 - file not found on server */
+    case CURLE_TFTP_PERM:             /* 69 - permission problem on server */
+    case CURLE_REMOTE_DISK_FULL:      /* 70 - out of disk space on server */
+    case CURLE_TFTP_ILLEGAL:          /* 71 - Illegal TFTP operation */
+    case CURLE_TFTP_UNKNOWNID:        /* 72 - Unknown transfer ID */
+    case CURLE_REMOTE_FILE_EXISTS:    /* 73 - File already exists */
+    case CURLE_TFTP_NOSUCHUSER:       /* 74 - No such user */
+    case CURLE_OBSOLETE75:            /* 75 - NOT IN USE since 7.82.0 */
+    case CURLE_OBSOLETE76:            /* 76 - NOT IN USE since 7.82.0 */
+    case CURLE_SSL_CACERT_BADFILE: /* 77 - could not load CACERT file, missing
+                                     or wrong format */
+    case CURLE_REMOTE_FILE_NOT_FOUND: /* 78 - remote file not found */
+    case CURLE_SSH:                   /* 79 - error from the SSH layer, somewhat
+                                        generic so the error message will be of
+                                        interest when this has happened */
+
+    case CURLE_AGAIN:              /* 81 - socket is not ready for send/recv,
+                                     wait till it's ready and try again (Added
+                                     in 7.18.2) */
+    case CURLE_SSL_CRL_BADFILE:    /* 82 - could not load CRL file, missing or
+                                     wrong format (Added in 7.19.0) */
+    case CURLE_SSL_ISSUER_ERROR:   /* 83 - Issuer check failed.  (Added in
+                                     7.19.0) */
+    case CURLE_FTP_PRET_FAILED:    /* 84 - a PRET command failed */
+    case CURLE_RTSP_CSEQ_ERROR:    /* 85 - mismatch of RTSP CSeq numbers */
+    case CURLE_RTSP_SESSION_ERROR: /* 86 - mismatch of RTSP Session Ids */
+    case CURLE_FTP_BAD_FILE_LIST:  /* 87 - unable to parse FTP file list */
+    case CURLE_CHUNK_FAILED:       /* 88 - chunk callback reported error */
+    case CURLE_NO_CONNECTION_AVAILABLE:  /* 89 - No connection available, the
+                                         session will be queued */
+    case CURLE_SSL_PINNEDPUBKEYNOTMATCH: /* 90 - specified pinned public key did
+                                         not match */
+    case CURLE_SSL_INVALIDCERTSTATUS:    /* 91 - invalid certificate status */
+    case CURLE_HTTP2_STREAM:       /* 92 - stream error in HTTP/2 framing layer
+                                    */
+    case CURLE_RECURSIVE_API_CALL: /* 93 - an api function was called from
+                                     inside a callback */
+    case CURLE_HTTP3:              /* 95 - An HTTP/3 layer problem */
+    case CURLE_QUIC_CONNECT_ERROR: /* 96 - QUIC connection error */
+    case CURLE_PROXY:              /* 97 - proxy handshake error */
+    case CURLE_SSL_CLIENTCERT:     /* 98 - client-side certificate required */
+    case CURLE_UNRECOVERABLE_POLL: /* 99 - poll/select returned fatal error */
+      return true;
+    default:
+      return false;
+  }
+}
+
+bool Curl::should_retry_request(CURLcode curl_code, long http_code) const {
+  if (curl_code != CURLE_OK) {
+    return should_retry_based_on_curl_code(curl_code);
+  } else {
+    return should_retry_based_on_http_status(http_code);
+  }
 }
 
 tuple<Status, optional<long>> Curl::last_http_status_code() {
@@ -657,12 +795,20 @@ Status Curl::check_curl_errors(
     return LOG_STATUS(
         Status_RestError("Error checking curl error; curl instance is null."));
 
+  if (curl_code != CURLE_OK) {
+    std::stringstream msg;
+    msg << "Error in libcurl " << operation
+        << " operation: libcurl error message '" << get_curl_errstr(curl_code)
+        << "; ";
+    return Status_RestError(msg.str());
+  }
+
   long http_code = 0;
   if (curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_code) != CURLE_OK)
     return LOG_STATUS(Status_RestError(
         "Error checking curl error; could not get HTTP code."));
 
-  if (curl_code != CURLE_OK || http_code >= 400) {
+  if (http_code >= 400) {
     // TODO: Should see if message has error data object
     std::stringstream msg;
     msg << "Error in libcurl " << operation
@@ -679,7 +825,6 @@ Status Curl::check_curl_errors(
         msg << "server response was empty.";
       }
     }
-
     return Status_RestError(msg.str());
   }
 
@@ -706,7 +851,7 @@ Status Curl::post_data(
     stats::Stats* const stats,
     const std::string& url,
     const SerializationType serialization_type,
-    const BufferList* data,
+    BufferList* data,
     Buffer* const returned_data,
     const std::string& res_uri) {
   struct curl_slist* headers;
@@ -716,7 +861,7 @@ Status Curl::post_data(
 
   CURLcode ret;
   headerData.uri = &res_uri;
-  auto st = make_curl_request(stats, url.c_str(), &ret, returned_data);
+  auto st = make_curl_request(stats, url.c_str(), &ret, data, returned_data);
   curl_slist_free_all(headers);
   RETURN_NOT_OK(st);
 
@@ -730,7 +875,7 @@ Status Curl::post_data(
     stats::Stats* const stats,
     const std::string& url,
     const SerializationType serialization_type,
-    const BufferList* data,
+    BufferList* data,
     Buffer* const returned_data,
     PostResponseCb&& cb,
     const std::string& res_uri) {
@@ -739,7 +884,7 @@ Status Curl::post_data(
 
   CURLcode ret;
   headerData.uri = &res_uri;
-  auto st = make_curl_request(stats, url.c_str(), &ret, std::move(cb));
+  auto st = make_curl_request(stats, url.c_str(), &ret, data, std::move(cb));
   curl_slist_free_all(headers);
   RETURN_NOT_OK(st);
 
@@ -751,7 +896,7 @@ Status Curl::post_data(
 
 Status Curl::post_data_common(
     const SerializationType serialization_type,
-    const BufferList* data,
+    BufferList* data,
     struct curl_slist** headers) {
   CURL* curl = curl_.get();
   if (curl == nullptr)
@@ -773,7 +918,7 @@ Status Curl::post_data_common(
       set_content_type(serialization_type, headers),
       curl_slist_free_all(*headers));
 
-  /* HTTP PUT please */
+  /* HTTP POST please */
   curl_easy_setopt(curl, CURLOPT_POST, 1L);
   curl_easy_setopt(
       curl, CURLOPT_READFUNCTION, buffer_list_read_memory_callback);
@@ -812,7 +957,7 @@ Status Curl::get_data(
 
   CURLcode ret;
   headerData.uri = &res_ns_uri;
-  auto st = make_curl_request(stats, url.c_str(), &ret, returned_data);
+  auto st = make_curl_request(stats, url.c_str(), &ret, nullptr, returned_data);
   curl_slist_free_all(headers);
   RETURN_NOT_OK(st);
 
@@ -851,7 +996,7 @@ Status Curl::options(
 
   CURLcode ret;
   headerData.uri = &res_ns_uri;
-  auto st = make_curl_request(stats, url.c_str(), &ret, returned_data);
+  auto st = make_curl_request(stats, url.c_str(), &ret, nullptr, returned_data);
   curl_slist_free_all(headers);
   RETURN_NOT_OK(st);
 
@@ -887,7 +1032,7 @@ Status Curl::delete_data(
 
   CURLcode ret;
   headerData.uri = &res_uri;
-  auto st = make_curl_request(stats, url.c_str(), &ret, returned_data);
+  auto st = make_curl_request(stats, url.c_str(), &ret, nullptr, returned_data);
 
   // Erase record in case of de-registered array
   std::unique_lock<std::mutex> rd_lck(*(headerData.redirect_uri_map_lock));
@@ -905,7 +1050,7 @@ Status Curl::patch_data(
     stats::Stats* const stats,
     const std::string& url,
     const SerializationType serialization_type,
-    const BufferList* data,
+    BufferList* data,
     Buffer* const returned_data,
     const std::string& res_uri) {
   struct curl_slist* headers;
@@ -913,7 +1058,7 @@ Status Curl::patch_data(
 
   CURLcode ret;
   headerData.uri = &res_uri;
-  auto st = make_curl_request(stats, url.c_str(), &ret, returned_data);
+  auto st = make_curl_request(stats, url.c_str(), &ret, data, returned_data);
   curl_slist_free_all(headers);
   RETURN_NOT_OK(st);
 
@@ -925,7 +1070,7 @@ Status Curl::patch_data(
 
 Status Curl::patch_data_common(
     const SerializationType serialization_type,
-    const BufferList* data,
+    BufferList* data,
     struct curl_slist** headers) {
   CURL* curl = curl_.get();
   if (curl == nullptr)
@@ -970,7 +1115,7 @@ Status Curl::put_data(
     stats::Stats* const stats,
     const std::string& url,
     const SerializationType serialization_type,
-    const BufferList* data,
+    BufferList* data,
     Buffer* const returned_data,
     const std::string& res_uri) {
   struct curl_slist* headers;
@@ -978,7 +1123,7 @@ Status Curl::put_data(
 
   CURLcode ret;
   headerData.uri = &res_uri;
-  auto st = make_curl_request(stats, url.c_str(), &ret, returned_data);
+  auto st = make_curl_request(stats, url.c_str(), &ret, data, returned_data);
   curl_slist_free_all(headers);
   RETURN_NOT_OK(st);
 
@@ -990,7 +1135,7 @@ Status Curl::put_data(
 
 Status Curl::put_data_common(
     const SerializationType serialization_type,
-    const BufferList* data,
+    BufferList* data,
     struct curl_slist** headers) {
   CURL* curl = curl_.get();
   if (curl == nullptr)
@@ -1030,5 +1175,5 @@ Status Curl::put_data_common(
 
   return Status::Ok();
 }
-}  // namespace sm
-}  // namespace tiledb
+
+}  // namespace tiledb::sm
