@@ -105,8 +105,9 @@ Status metadata_from_capnp(
 
   for (size_t i = 0; i < num_entries; i++) {
     auto entry_reader = entries_reader[i];
-    auto key = std::string{std::string_view{
-        entry_reader.getKey().cStr(), entry_reader.getKey().size()}};
+    auto entry_key = entry_reader.getKey();
+    auto key =
+        std::string{std::string_view{entry_key.cStr(), entry_key.size()}};
     Datatype type = datatype_enum(entry_reader.getType());
     uint32_t value_num = entry_reader.getValueNum();
 
